@@ -8,11 +8,16 @@ GCC_ARGS = [
     '-Werror',
     '-Wall',
     '-Wextra',
+    '-ftrapv',                    # catch signed overflow on addition, subtraction, multiplication operations
     '-fsanitize=address',
     '-fsanitize=undefined',
     '-fno-sanitize-recover=all',  # for RE in case of UB
 ]
 
+LINK_ARGS = [
+    '-lm',
+]
+
 VALGRIND_ARGS = [
     'valgrind',
     '--leak-check=full',
@@ -43,7 +48,7 @@ def compile_solution(directory):
 def compile_c(workdir, files):
     filenames = [path.relative_to(workdir) for path in files]
 
-    command = GCC_ARGS + filenames
+    command = GCC_ARGS + filenames + LINK_ARGS
     p = subprocess.run(command, cwd=workdir)
 
     if p.returncode != 0:
